package be.iminds.ilabt.jfed.util;

import javax.annotation.Nonnull;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Creates ThreadFactory for threadpools with custom thread names.
 *
 * Advantage over ThreadPoolFactory is that it numbers the pool itself for each name. This can be useful when debugging.
 */
public class ThreadFactoryUtil {
    static class ThreadFactoryUtilThreadFactory implements ThreadFactory {
        private final ThreadGroup group;
        private final AtomicInteger threadNumber = new AtomicInteger(1);
        private final @Nonnull String namePrefix;
        private final Boolean forceDaemon;

        ThreadFactoryUtilThreadFactory(@Nonnull String poolId, int poolNumber, Boolean forceDaemon) {
            SecurityManager s = System.getSecurityManager();
            this.forceDaemon = forceDaemon;
            this.group = (s != null) ? s.getThreadGroup() :
                    Thread.currentThread().getThreadGroup();
            this.namePrefix = poolId + "-" + poolNumber + "-thread-";
        }

        @Override
        public Thread newThread(@Nonnull Runnable r) {
            Thread t = new Thread(group, r,
                    namePrefix + threadNumber.getAndIncrement(),
                    0);
            if (forceDaemon != null) {
                t.setDaemon(forceDaemon);
            } else {
                if (t.isDaemon())
                    t.setDaemon(false);
            }
            if (t.getPriority() != Thread.NORM_PRIORITY)
                t.setPriority(Thread.NORM_PRIORITY);
            return t;
        }
    }

    private static Map<String, AtomicInteger> poolIdUsageCount = new ConcurrentHashMap<>();

    public static ThreadFactory getFactory(@Nonnull String poolId) {
        return getFactory(poolId, null);
    }
    public static ThreadFactory getFactory(@Nonnull String poolId, Boolean forceDaemon) {
        AtomicInteger poolNumber = poolIdUsageCount.computeIfAbsent(poolId, (key) -> new AtomicInteger(1) );
        return new ThreadFactoryUtilThreadFactory(poolId, poolNumber.getAndIncrement(), forceDaemon);
    }
}
